--[[
    Guild Mailbox
  ]]--

--configuration
GUILDMAILBOX_CONFIG = {}
GUILDMAILBOX_CONFIG["GuildMailAddress"] = "Guild"

GUILDMAILBOX_MAILQUEUE = Queue_new()

local timeSinceLastUpdate = 0
local ticksUntilUpdate = 50

local debug = false

local currentRecipient;
local readyToSendNextMessage = true

--slash commands
SLASH_GUILDMAILBOX1 = "/guildmailbox";
-----------------------------------------------------------------------------------
--
-- Simple print function, that outputs strings to the currently active chat frame
--
-----------------------------------------------------------------------------------
function Print(message) 
	DEFAULT_CHAT_FRAME:AddMessage(message)
end

-----------------------------------------------------------------------------------
--
-- Debug function, only outputs message if debug is set to true
--
-----------------------------------------------------------------------------------
function Debug(message)
	if (debug) then
		Print(message)
	end
end

---------------------------------------------------------------------------------
--
-- Break a chat command into its command and variable parts (i.e. "debug on"
-- breaks into command = "debug" and variable = "on"
-- 
---------------------------------------------------------------------------------
function GuildMailbox_ParseCommand(msg)
    firstSpace = string.find(msg, " ", 1, true);
    if (firstSpace) then
        local command = string.sub(msg, 1, firstSpace - 1);
        local var  = string.sub(msg, firstSpace + 1);
        return command, var
    else
        return msg, nil;
    end
end

-----------------------------------------------------------------------------------
--
-- process slash commands
--
-----------------------------------------------------------------------------------
function GuildMailbox_SlashCommand(message) 
	local command, argument = GuildMailbox_ParseCommand(message)
	if (command == "clear") then
		GUILDMAILBOX_MAILQUEUE = Queue_new()
		Print("Cleared mail queue.")
    elseif (command == "debug") then
		-- switch debug
		debug = not debug
		if (debug) then
			Print("Debugging enabled.")
		else 
			Print("Debugging disabled.")		
		end
	elseif (command == "address") then
		if (not argument) then
			Print(string.format("Guild mail address is '%s'.", GUILDMAILBOX_CONFIG["GuildMailAddress"]))
		elseif (string.find(argument, "%a")) then
			GUILDMAILBOX_CONFIG["GuildMailAddress"] = argument
			Print(string.format("Set guild mail address to '%s'.", argument))
		else
			Print(string.format("Invalid guild mail address '%s'.", argument))
		end
	else 
		Print("Currently implemented commands:\n" ..
							 "/guildmailbox clear   -- clears mailing queue.\n" ..
							 "/guildmailbox address newaddress -- changes special guild address.\n" ..							 
							 "/guildmailbox debug   -- toggles debugging on/off (with debugging on, no real mails are sent).\n")
	end
end

-----------------------------------------------------------------------------------
--
-- Handle the loading stuff when the plugin first loads
--
-----------------------------------------------------------------------------------
function GuildMailbox_OnLoad()   
	-- register for all mail related events
    this:RegisterEvent("MAIL_FAILED")
    this:RegisterEvent("MAIL_SHOW")
    this:RegisterEvent("MAIL_CLOSED")
    this:RegisterEvent("MAIL_SEND_SUCCESS")
	
    -- hook the sendMail function so we can take it over to do our bidding
    GuildMailbox_HookSendMail(true) 
	
	SlashCmdList["GUILDMAILBOX"] = GuildMailbox_SlashCommand
	
	Print("|cff8080ffGuild Mailbox|r Version 0.0.4  /guildmailbox shows availabe commands."
)
end

-----------------------------------------------------------------------------------
--
-- Checks for events 
--
-----------------------------------------------------------------------------------
function GuildMailbox_OnEvent(event)
    if (event == "MAIL_SEND_SUCCESS") then
		if (currentRecipient ~= nil) then
			local message = string.format("Mail sent successfully to '%s'. [%d mails left.]"
, currentRecipient, Queue_size(GUILDMAILBOX_MAILQUEUE))
			Print(message)
			
			currentRecipient = nil
		end
        readyToSendNextMessage = true
		
	elseif (event == "MAIL_FAILED") then
		if (currentRecipient ~= nil) then
			local message = string.format("Error while sending to '%s'. [%d mails left]", currentRecipient, Queue_size(GUILDMAILBOX_MAILQUEUE))
			Print(message)
			
			currentRecipient = nil
		end
		readyToSendNextMessage = true
		
    elseif (event == "MAIL_CLOSED") then
		if (GuildMailboxFrame:IsVisible()) then
			if (not Queue_empty(GUILDMAILBOX_MAILQUEUE)) then
				Print("Mailbox closed. Pausing mail delivery.")
			end
		
			-- hide script frame to avoid unecessary update calls
			GuildMailboxFrame:Hide()
			GuildMailboxForm:Hide()
		end
		
	elseif (event == "MAIL_SHOW") then
		-- show frame to get update calls again
		GuildMailboxFrame:Show()
    end
end

-----------------------------------------------------------------------------------
--
-- Update
--
-----------------------------------------------------------------------------------
function GuildMailbox_OnUpdate(elapsed)
	timeSinceLastUpdate = timeSinceLastUpdate + 1
	
	-- to debug without sending really mail
	if (debug) then
		if (timeSinceLastUpdate > 5) then
			GuildMailbox_OnEvent("Mail sent successfully to '%s'. [%d mail(s) left.]"
)
		end
	end

	-- send mail if previous mail finished sending
	if (timeSinceLastUpdate > ticksUntilUpdate) then
		--Debug("Ready to send message: " .. asText(readyToSendNextMessage) .. " Queue empty: " .. asText(Queue_empty(GUILDMAILBOX_MAILQUEUE)))
		if (readyToSendNextMessage and not Queue_empty(GUILDMAILBOX_MAILQUEUE)) then
			local mail = Queue_pop(GUILDMAILBOX_MAILQUEUE)
				
			if (not debug) then
				SetSendMailMoney(mail[4])
				SendMail(mail[1], mail[2], mail[3])
			end 
			
			currentRecipient = mail[1]
			readyToSendNextMessage = false
		end
		timeSinceLastUpdate = 0
	end
	
end

---------------------------------------------------------------------------------
--
-- Hook us onto the original sendmail() function
-- 
---------------------------------------------------------------------------------
function GuildMailbox_HookSendMail(toggle)
    if (toggle) then
        originalSendMail = SendMailFrame_SendMail
        SendMailFrame_SendMail = GuildMailbox_SendMail
    else
        SendMailFrame_SendMail = GuildMailbox_originalSendMail
        GuildMailbox_originalSendMail = nil
    end
end


-----------------------------------------------------------------------------------
--
-- Overloads the original SendMailFrame_SendMail() function
--
-----------------------------------------------------------------------------------
function GuildMailbox_SendMail()
	local body = SendMailBodyEditBox:GetText()
	local subject = SendMailSubjectEditBox:GetText()
	local recipient = SendMailNameEditBox:GetText()
	
	if (recipient == GUILDMAILBOX_CONFIG["GuildMailAddress"]) then
		GuildMailboxForm:Show()
	else
		SendMail(recipient, subject, body)
	end
end

-----------------------------------------------------------------------------------
--
-- Debug function to view any lua object as text
--
-----------------------------------------------------------------------------------
function asText(obj)

    local visitRef = {}
    visitRef.n = 0

    asTxRecur = function(obj, asIndex)
        if type(obj) == "table" then
            if visitRef[obj] then
                return "@"..visitRef[obj]
            end
            visitRef.n = visitRef.n +1
            visitRef[obj] = visitRef.n

            local begBrac, endBrac
            if asIndex then
                begBrac, endBrac = "[{", "}]"
            else
                begBrac, endBrac = "{", "}"
            end
            local t = begBrac
            local k, v = nil, nil
            repeat
                k, v = next(obj, k)
                if k ~= nil then
                    if t > begBrac then
                        t = t..", "
                    end
                    t = t..asTxRecur(k, 1).."="..asTxRecur(v)
                end
            until k == nil
            return t..endBrac
        else
            if asIndex then
                -- we're on the left side of an "="
                if type(obj) == "string" then
                    return obj
                else
                    return "["..obj.."]"
                end
            else
                -- we're on the right side of an "="
                if type(obj) == "string" then
                    return '"'..obj..'"'
                else
                    return tostring(obj)
                end
            end
        end
    end -- asTxRecur

    return asTxRecur(obj)
end -- asText